(function () {
  var lastFocusElement;

  function hasClass(elem, classes) {
    if (typeof classes === 'string') {
      classes = [classes];
    }
    var elem_classes = elem.className.split(' ');

    for (var c = 0; c < classes.length; c++) {
      if (classes[c][0] === '!') {
        if (elem_classes.indexOf(classes[c].slice(1)) >= 0) {
          return false;
        }
      } else {
        if (elem_classes.indexOf(classes[c]) < 0) {
          return false;
        }  
      }
    }

    return true;
  }

  function forChildren(elem, func) {
    if (elem && elem.children && elem.children.length) {
      for (var c = 0; c < elem.children.length; c++) {
        if (elem.children[c] !== elem) {
          func(elem.children[c], c, elem.children);
        }
      }
    }
  }

  function forSiblings(elem, func) {
    if (elem) {
      forChildren(elem.parentElement, func);
    }
  }

  function getHeirarchy(elem) {
    var heirarchy = [];

    while (elem) {
      heirarchy.push(elem);
      elem = elem.parentElement;
    }

    return heirarchy;
  }

  function findMenuLink (elem) {
    if (elem) {
      if (hasClass(elem, 'menu-link')) {
        return elem;
      }

      for (var child = elem.firstElementChild; child; child = child.nextElementSibling) {
        if (!hasClass(child, 'menu')) {
          var link = findMenuLink(child);

          if (link) {
            return link;
          }
        }
      }  
    }

    return null;
  }

  function getNextSiblingMenuLink (elem) {
    if (hasClass(elem, 'menu-link')) {
      elem = elem.parentElement;
    }

    for (var m = elem.nextElementSibling; m; m = m.nextElementSibling) {
      var link = findMenuLink(m);

      if (link) {
        return link;
      }
    }

    return null;
  }

  function getPrevSiblingMenuLink (elem) {
    if (hasClass(elem, 'menu-link')) {
      elem = elem.parentElement;
    }

    for (var m = elem.previousElementSibling; m; m = m.previousElementSibling) {
      var link = findMenuLink(m);

      if (link) {
        return link;
      }
    }

    return null;
  }

  function getFirstChildMenuLink (elem) {
    if (hasClass(elem, 'menu-link')) {
      elem = elem.parentElement;
    }

    for (var m = elem.firstElementChild; m; m = m.nextElementSibling) {
      if (hasClass(m, 'menu')) {
        var link = findMenuLink(m);

        if (link) {
          return link;
        }
      }
    }

    return null;
  }

  function getAncestorMenuLink (elem) {
    var ret;

    for (var p of getHeirarchy(elem).reverse()) {
      if (hasClass(p, 'menu')) {
        forSiblings(p, function (sib) {
          if (hasClass(sib, ['menu-link', 'has-submenu', '!hamburger-menu'])) {
            ret = sib;
          }
        });
      } else if (hasClass(p, ['menu-link', '!hamburger-menu'])) {
        ret = p;
      }

      if (ret) {
        return ret;
      }
    }

    return null;
  }

  function activateMenu(elem) {
    var touched = [],
      close = [].slice.call(document.querySelectorAll('.dropdown-menus, .dropdown-menus .block-menu, .dropdown-menus .menu, .dropdown-menus .menu-item.has-submenu, .dropdown-menus .menu-link.has-submenu')),
      canOpen = true;

    lastFocusElement = false;

    if (elem) {
      getHeirarchy(elem).forEach(function (elem) {
        if (hasClass(elem, 'menu')) {
          if (canOpen) {
            elem.classList.add('open');
          } else {
            canOpen = true;
            elem.classList.remove('open');
          }
  
          elem.setAttribute('aria-hidden', false);
          touched.push(elem);
  
          forSiblings(elem, function (sib) {
            touched.push(sib);
  
            if (hasClass(sib, ['menu-link', 'has-submenu'])) {
              lastFocusElement = lastFocusElement || sib;
              sib.classList.add('open');
            }
          });
        } else if (hasClass(elem, ['menu-link', 'has-submenu', '!open'])) {
          canOpen = true;
          lastFocusElement = lastFocusElement || elem;
          elem.classList.add('open');
          touched.push(elem);
          forSiblings(elem, function (sib) {
            touched.push(sib);
  
            if (hasClass(sib, 'menu')) {
              sib.classList.remove('open');
              sib.setAttribute('aria-hidden', false);
            }
          });
        } else if (hasClass(elem, ['menu-link'])) {
          canOpen = false;
          elem.classList.remove('open');
          touched.push(elem);
        } else if (hasClass(elem, ['menu-item'])) {
          if (canOpen) {
            elem.classList.add('open');
          } else {
            elem.classList.remove('open');
          }
          touched.push(elem);
        } else if (hasClass(elem, ['block-menu']) || hasClass(elem, ['dropdown-menus'])) {
          elem.classList.add('open');
          touched.push(elem);
        }
      });
    }

    close = close.filter(function (elem) { return touched.indexOf(elem) < 0; });

    close.forEach(function (elem) {
      elem.classList.remove('open');

      if (hasClass(elem, 'menu')) {
        elem.setAttribute('aria-hidden', true);
      }
    });

    if (lastFocusElement) {
      if (hasClass(lastFocusElement, ['has-submenu', 'open'])) {
        lastFocusElement = getFirstChildMenuLink(lastFocusElement) || lastFocusElement;
      }

      lastFocusElement.focus();
    }

    return touched.length + close.length;
  }

  function clickHandler (e) {
    activateMenu(e.target);

    if (hasClass(e.target, ['menu-link', 'has-submenu'])) {
      e.preventDefault();
      e.stopPropagation();
    }
  }

  function keydownHandler (e) {
    var link;

    if (e.keyCode === 27 && ((link = getAncestorMenuLink(document.activeElement)), activateMenu())) {
      e.preventDefault();
      e.stopPropagation();

      if (link) {
        link.focus();
      }
    } else if (e.keyCode == 32 && hasClass(e.target, ['menu-link', 'has-submenu']) && activateMenu(e.target)) {
      e.preventDefault();
      e.stopPropagation();
    } else if ((e.keyCode === 38 || e.keyCode === 40) && hasClass(e.target, ['menu-link', 'has-submenu', '!open']) && activateMenu(e.target)) {
      e.preventDefault();
      e.stopPropagation();
    } else if (hasClass(e.target, ['menu-link'])) {
      if (e.keyCode === 38 || e.keyCode === 40) {
        var elem = e.keyCode === 38 ? getPrevSiblingMenuLink(e.target) : getNextSiblingMenuLink(e.target);

        if (elem) {
          elem.focus();
        }

        e.preventDefault();
        e.stopPropagation();
      } else if (e.keyCode === 37 || e.keyCode === 39) {
        var anc = getAncestorMenuLink(e.target);

        var elem = e.keyCode === 37 ? getPrevSiblingMenuLink(anc) : getNextSiblingMenuLink(anc);

        if (e.target === anc) {
          elem.focus();
        } else {
          activateMenu(elem);
        }

        e.preventDefault();
        e.stopPropagation();
      }
    }
  }

  function keyEater (e) {
    if ([13, 32, 37, 38, 39, 40].indexOf(e.keyCode) >= 0 && hasClass(e.target, ['menu-link', 'has-submenu'])) {
      e.preventDefault();
      e.stopPropagation();
    }
  }

  Drupal.behaviors.soahMenus = {
    attach: function () {
      var elems = document.querySelectorAll('.dropdown-menus .menu-link.has-submenu');

      for (var c = 0; c < elems.length; c++) {
        elems[c].classList.add('menu-handler');
        elems[c].setAttribute('aria-haspopup', true);

        forSiblings(elems[c], function (sib) {
          if (hasClass(sib, 'menu') && !sib.getAttribute('aria-hidden')) {
            sib.setAttribute('aria-hidden', true);
          }
        });
      }

    }
  };

  window.addEventListener('click', clickHandler);
  window.addEventListener('keydown', keydownHandler);
  window.addEventListener('keyup', keyEater);
  window.addEventListener('keypress', keyEater);
  Drupal.behaviors.soahMenus.attach();
})();
